const { app, BrowserWindow, ipcMain } = require('electron');
const path = require('path');
const fs = require('fs').promises;
const isDev = process.env.NODE_ENV === 'development';

let mainWindow;
const DATA_DIR = path.join(app.getPath('userData'), 'data');
const GROUPS_FILE = path.join(DATA_DIR, 'groups.json');

// 确保数据目录存在
async function ensureDataDir() {
  try {
    await fs.mkdir(DATA_DIR, { recursive: true });
    try {
      await fs.access(GROUPS_FILE);
    } catch {
      await fs.writeFile(GROUPS_FILE, JSON.stringify({ groups: [] }));
    }
  } catch (err) {
    console.error('Failed to create data directory:', err);
  }
}

function createWindow() {
  console.log('Creating window...');
  
  mainWindow = new BrowserWindow({
    width: 1200,
    height: 800,
    show: false,
    webPreferences: {
      nodeIntegration: false,
      contextIsolation: true,
      preload: path.join(__dirname, 'preload.js'),
      enableRemoteModule: false,
      sandbox: true
    }
  });

  const devServerUrl = 'http://localhost:5173';
  
  if (isDev) {
    console.log('Loading URL in dev mode:', devServerUrl);
    mainWindow.loadURL(devServerUrl).catch(err => {
      console.error('Failed to load dev server:', err);
      app.quit();
    });
    mainWindow.webContents.openDevTools();
  } else {
    const indexPath = path.join(__dirname, '../dist/index.html');
    console.log('Loading file in prod mode:', indexPath);
    mainWindow.loadFile(indexPath).catch(err => {
      console.error('Failed to load index file:', err);
      app.quit();
    });
  }

  mainWindow.webContents.on('did-fail-load', (event, errorCode, errorDescription) => {
    console.error('Failed to load:', errorCode, errorDescription);
  });

  mainWindow.once('ready-to-show', () => {
    console.log('Window ready to show');
    mainWindow.show();
  });

  mainWindow.on('closed', () => {
    console.log('Window closed');
    mainWindow = null;
  });
}

// 处理保存文件
ipcMain.handle('save-file', async (event, content) => {
  try {
    const { filePath } = await dialog.showSaveDialog({
      title: 'Save Markdown File',
      defaultPath: path.join(app.getPath('documents'), 'untitled.md'),
      filters: [
        { name: 'Markdown Files', extensions: ['md'] },
        { name: 'All Files', extensions: ['*'] }
      ]
    });

    if (filePath) {
      await fs.writeFile(filePath, content, 'utf8');
      return { success: true, filePath };
    }
    return { success: false, error: 'No file selected' };
  } catch (error) {
    return { success: false, error: error.message };
  }
});

// 处理导出 PDF
ipcMain.handle('export-pdf', async (event) => {
  try {
    const win = BrowserWindow.fromWebContents(event.sender);
    
    const { filePath } = await dialog.showSaveDialog(win, {
      title: 'Export PDF',
      defaultPath: path.join(app.getPath('documents'), 'untitled.pdf'),
      filters: [
        { name: 'PDF Files', extensions: ['pdf'] }
      ]
    });

    if (!filePath) {
      return { success: false, error: 'No file selected' };
    }

    // 获取预览内容的 HTML
    const previewHTML = await event.sender.executeJavaScript(`
      document.querySelector('.preview-for-pdf').innerHTML
    `);

    // 创建一个临时窗口来渲染 PDF
    const tempWin = new BrowserWindow({
      width: 800,
      height: 600,
      show: false
    });

    // 加载一个包含预览内容的临时 HTML
    await tempWin.loadURL(`data:text/html;charset=utf-8,
      <!DOCTYPE html>
      <html>
        <head>
          <style>
            body {
              font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif;
              line-height: 1.6;
              color: #333;
              padding: 20px;
            }
            pre {
              background-color: #f8f8f8;
              padding: 1rem;
              border-radius: 4px;
              overflow-x: auto;
            }
            code {
              background-color: #f8f8f8;
              padding: 0.2em 0.4em;
              border-radius: 3px;
              font-family: 'SFMono-Regular', Consolas, 'Liberation Mono', Menlo, Courier, monospace;
            }
            blockquote {
              margin: 1rem 0;
              padding-left: 1rem;
              border-left: 4px solid #e0e0e0;
              color: #666;
            }
            table {
              border-collapse: collapse;
              width: 100%;
              margin: 1rem 0;
            }
            th, td {
              border: 1px solid #e0e0e0;
              padding: 8px;
              text-align: left;
            }
            th {
              background-color: #f8f8f8;
            }
          </style>
        </head>
        <body>
          ${previewHTML}
        </body>
      </html>
    `);

    // 导出为 PDF
    const pdfData = await tempWin.webContents.printToPDF({
      printBackground: true,
      margin: {
        top: 36,
        bottom: 36,
        left: 36,
        right: 36
      }
    });

    // 保存 PDF 文件
    await fs.writeFile(filePath, pdfData);

    // 关闭临时窗口
    tempWin.close();

    return { success: true, filePath };
  } catch (error) {
    console.error('Error exporting PDF:', error);
    return { success: false, error: error.message };
  }
});

// 文件系统操作处理
async function handleFileSystem() {
  // 获取所有分组
  ipcMain.handle('get-groups', async () => {
    try {
      const data = await fs.readFile(GROUPS_FILE, 'utf-8');
      return JSON.parse(data);
    } catch (err) {
      console.error('Failed to read groups:', err);
      return { groups: [] };
    }
  });

  // 创建新分组
  ipcMain.handle('create-group', async (event, groupName) => {
    try {
      const data = await fs.readFile(GROUPS_FILE, 'utf-8');
      const groups = JSON.parse(data);
      const newGroup = {
        id: Date.now().toString(),
        name: groupName,
        files: []
      };
      groups.groups.push(newGroup);
      await fs.writeFile(GROUPS_FILE, JSON.stringify(groups));
      return newGroup;
    } catch (err) {
      console.error('Failed to create group:', err);
      throw err;
    }
  });

  // 删除分组
  ipcMain.handle('delete-group', async (event, groupId) => {
    try {
      const data = await fs.readFile(GROUPS_FILE, 'utf-8');
      const groups = JSON.parse(data);
      groups.groups = groups.groups.filter(g => g.id !== groupId);
      await fs.writeFile(GROUPS_FILE, JSON.stringify(groups));
      return true;
    } catch (err) {
      console.error('Failed to delete group:', err);
      throw err;
    }
  });

  // 创建新文件
  ipcMain.handle('create-file', async (event, groupId, fileName) => {
    try {
      const data = await fs.readFile(GROUPS_FILE, 'utf-8');
      const groups = JSON.parse(data);
      const group = groups.groups.find(g => g.id === groupId);
      if (!group) throw new Error('Group not found');
      
      const newFile = {
        id: Date.now().toString(),
        name: fileName,
        content: ''
      };
      
      group.files.push(newFile);
      await fs.writeFile(GROUPS_FILE, JSON.stringify(groups));
      return newFile;
    } catch (err) {
      console.error('Failed to create file:', err);
      throw err;
    }
  });

  // 删除文件
  ipcMain.handle('delete-file', async (event, groupId, fileId) => {
    try {
      const data = await fs.readFile(GROUPS_FILE, 'utf-8');
      const groups = JSON.parse(data);
      const group = groups.groups.find(g => g.id === groupId);
      if (!group) throw new Error('Group not found');
      
      group.files = group.files.filter(f => f.id !== fileId);
      await fs.writeFile(GROUPS_FILE, JSON.stringify(groups));
      return true;
    } catch (err) {
      console.error('Failed to delete file:', err);
      throw err;
    }
  });

  // 保存文件内容
  ipcMain.handle('save-file-content', async (event, fileId, content) => {
    try {
      const data = await fs.readFile(GROUPS_FILE, 'utf-8');
      const groups = JSON.parse(data);
      let fileFound = false;
      
      for (const group of groups.groups) {
        const file = group.files.find(f => f.id === fileId);
        if (file) {
          file.content = content;
          fileFound = true;
          break;
        }
      }
      
      if (!fileFound) throw new Error('File not found');
      await fs.writeFile(GROUPS_FILE, JSON.stringify(groups));
      return true;
    } catch (err) {
      console.error('Failed to save file:', err);
      throw err;
    }
  });

  // 加载文件内容
  ipcMain.handle('load-file-content', async (event, fileId) => {
    try {
      const data = await fs.readFile(GROUPS_FILE, 'utf-8');
      const groups = JSON.parse(data);
      
      for (const group of groups.groups) {
        const file = group.files.find(f => f.id === fileId);
        if (file) {
          return file.content;
        }
      }
      
      throw new Error('File not found');
    } catch (err) {
      console.error('Failed to load file:', err);
      throw err;
    }
  });
}

app.whenReady().then(async () => {
  await ensureDataDir();
  await handleFileSystem();
  createWindow();

  app.on('activate', () => {
    if (BrowserWindow.getAllWindows().length === 0) {
      createWindow();
    }
  });
});

app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit();
  }
});
