import React, { useState, useRef, useCallback } from 'react';
import Split from 'react-split';
import Editor from '@monaco-editor/react';
import { marked } from 'marked';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import {
  faBold,
  faItalic,
  faStrikethrough,
  faCode,
  faFileCode,
  faLink,
  faImage,
  faListUl,
  faListOl,
  faQuoteRight,
  faTable,
  faMinus,
  faColumns,
  faEdit,
  faEye,
  faSave,
  faFilePdf
} from '@fortawesome/free-solid-svg-icons';
import Sidebar from './components/Sidebar';
import './components/Sidebar.css';

type ViewMode = 'split' | 'edit' | 'preview';

interface Document {
  id: string;
  groupId: string;
  name: string;
  content: string;
}

const App: React.FC = () => {
  const editorRef = useRef<any>(null);
  const [markdown, setMarkdown] = useState<string>('');
  const [viewMode, setViewMode] = useState<ViewMode>('split');
  const [isSidebarOpen, setIsSidebarOpen] = useState(true);
  const [currentDocument, setCurrentDocument] = useState<Document | null>(null);

  const handleEditorDidMount = (editor: any) => {
    editorRef.current = editor;
  };

  const handleEditorChange = (value: string | undefined) => {
    setMarkdown(value || '');
    if (currentDocument) {
      window.electronAPI.saveFile(currentDocument.id, value || '');
    }
  };

  const insertAtCursor = (before: string, after: string = '') => {
    const editor = editorRef.current;
    if (!editor) return;

    const selection = editor.getSelection();
    const selectedText = editor.getModel().getValueInRange(selection);
    const newText = selectedText ? before + selectedText + after : before + after;

    editor.executeEdits('', [{
      range: selection,
      text: newText,
      forceMoveMarkers: true
    }]);

    if (!selectedText) {
      const newPosition = editor.getPosition();
      editor.setPosition({
        lineNumber: newPosition.lineNumber,
        column: newPosition.column - after.length
      });
    }
    editor.focus();
  };

  const handleToolbarAction = useCallback((action: string) => {
    switch (action) {
      case 'bold':
        insertAtCursor('**', '**');
        break;
      case 'italic':
        insertAtCursor('*', '*');
        break;
      case 'strikethrough':
        insertAtCursor('~~', '~~');
        break;
      case 'code':
        insertAtCursor('`', '`');
        break;
      case 'codeBlock':
        insertAtCursor('```\n', '\n```');
        break;
      case 'link':
        insertAtCursor('[', '](url)');
        break;
      case 'image':
        insertAtCursor('![alt text](', ')');
        break;
      case 'bulletList':
        insertAtCursor('- ');
        break;
      case 'numberedList':
        insertAtCursor('1. ');
        break;
      case 'quote':
        insertAtCursor('> ');
        break;
      case 'table':
        insertAtCursor('| Header | Header |\n| --- | --- |\n| Cell | Cell |');
        break;
      case 'hr':
        insertAtCursor('---\n');
        break;
      case 'save':
        break;
      case 'export':
        break;
    }
  }, []);

  const handleDocumentSelect = async (groupId: string, fileId: string) => {
    try {
      const content = await window.electronAPI.loadFile(fileId);
      setCurrentDocument({
        id: fileId,
        groupId,
        name: '', // 这个值会从 Sidebar 组件传过来
        content
      });
      setMarkdown(content);
    } catch (error) {
      console.error('Failed to load document:', error);
    }
  };

  const toggleSidebar = () => {
    setIsSidebarOpen(!isSidebarOpen);
  };

  return (
    <div className="app">
      <Sidebar 
        isOpen={isSidebarOpen} 
        onToggle={() => setIsSidebarOpen(!isSidebarOpen)}
        onDocumentSelect={handleDocumentSelect}
        currentDocumentId={currentDocument?.id}
      />
      <div className="main-content">
        <div className="toolbar">
          <div className="toolbar-group">
            <button onClick={() => handleToolbarAction('bold')} title="Bold">
              <FontAwesomeIcon icon={faBold} />
            </button>
            <button onClick={() => handleToolbarAction('italic')} title="Italic">
              <FontAwesomeIcon icon={faItalic} />
            </button>
            <button onClick={() => handleToolbarAction('strikethrough')} title="Strikethrough">
              <FontAwesomeIcon icon={faStrikethrough} />
            </button>
          </div>

          <div className="toolbar-group">
            <button onClick={() => handleToolbarAction('code')} title="Code">
              <FontAwesomeIcon icon={faCode} />
            </button>
            <button onClick={() => handleToolbarAction('codeBlock')} title="Code Block">
              <FontAwesomeIcon icon={faFileCode} />
            </button>
          </div>

          <div className="toolbar-group">
            <button onClick={() => handleToolbarAction('link')} title="Link">
              <FontAwesomeIcon icon={faLink} />
            </button>
            <button onClick={() => handleToolbarAction('image')} title="Image">
              <FontAwesomeIcon icon={faImage} />
            </button>
          </div>

          <div className="toolbar-group">
            <button onClick={() => handleToolbarAction('bulletList')} title="Bullet List">
              <FontAwesomeIcon icon={faListUl} />
            </button>
            <button onClick={() => handleToolbarAction('numberedList')} title="Numbered List">
              <FontAwesomeIcon icon={faListOl} />
            </button>
            <button onClick={() => handleToolbarAction('quote')} title="Quote">
              <FontAwesomeIcon icon={faQuoteRight} />
            </button>
          </div>

          <div className="toolbar-group">
            <button onClick={() => handleToolbarAction('table')} title="Table">
              <FontAwesomeIcon icon={faTable} />
            </button>
            <button onClick={() => handleToolbarAction('hr')} title="Horizontal Rule">
              <FontAwesomeIcon icon={faMinus} />
            </button>
          </div>

          <div className="toolbar-group">
            <button
              onClick={() => setViewMode('split')}
              className={viewMode === 'split' ? 'active' : ''}
              title="Split View"
            >
              <FontAwesomeIcon icon={faColumns} />
            </button>
            <button
              onClick={() => setViewMode('edit')}
              className={viewMode === 'edit' ? 'active' : ''}
              title="Edit Mode"
            >
              <FontAwesomeIcon icon={faEdit} />
            </button>
            <button
              onClick={() => setViewMode('preview')}
              className={viewMode === 'preview' ? 'active' : ''}
              title="Preview Mode"
            >
              <FontAwesomeIcon icon={faEye} />
            </button>
            <button onClick={() => handleToolbarAction('save')} title="Save">
              <FontAwesomeIcon icon={faSave} />
            </button>
            <button onClick={() => handleToolbarAction('export')} title="Export PDF">
              <FontAwesomeIcon icon={faFilePdf} />
            </button>
          </div>
        </div>
        {viewMode === 'split' && (
          <Split
            className="split-container"
            sizes={[50, 50]}
            minSize={200}
            gutterSize={8}
            snapOffset={0}
          >
            <div className="editor-container">
              <Editor
                height="100%"
                defaultLanguage="markdown"
                value={markdown}
                onChange={handleEditorChange}
                onMount={handleEditorDidMount}
                options={{
                  minimap: { enabled: false },
                  fontSize: 14,
                  wordWrap: 'on',
                  lineNumbers: 'on',
                  renderWhitespace: 'none',
                  scrollBeyondLastLine: false,
                }}
              />
            </div>
            <div
              className="preview-container markdown-preview"
              dangerouslySetInnerHTML={{ __html: marked(markdown) }}
            />
          </Split>
        )}
        {viewMode === 'edit' && (
          <div className="editor-container">
            <Editor
              height="100%"
              defaultLanguage="markdown"
              value={markdown}
              onChange={handleEditorChange}
              onMount={handleEditorDidMount}
              options={{
                minimap: { enabled: false },
                fontSize: 14,
                wordWrap: 'on',
                lineNumbers: 'on',
                renderWhitespace: 'none',
                scrollBeyondLastLine: false,
              }}
            />
          </div>
        )}
        {viewMode === 'preview' && (
          <div
            className="preview-container markdown-preview"
            dangerouslySetInnerHTML={{ __html: marked(markdown) }}
          />
        )}
      </div>
    </div>
  );
};

export default App;
