import React, { useState, useEffect } from 'react';
import { 
  faChevronLeft, 
  faChevronRight,
  faChevronDown,
  faFolder,
  faFolderPlus,
  faFile,
  faPlus,
  faTrash
} from '@fortawesome/free-solid-svg-icons';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import './Sidebar.css';
import ConfirmDialog from './ConfirmDialog';

interface SidebarProps {
  isOpen: boolean;
  onToggle: () => void;
  onDocumentSelect: (groupId: string, fileId: string) => void;
  currentDocumentId: string | undefined;
}

interface Group {
  id: string;
  name: string;
  files: File[];
}

interface File {
  id: string;
  name: string;
  content: string;
}

const Sidebar: React.FC<SidebarProps> = ({ 
  isOpen, 
  onToggle, 
  onDocumentSelect,
  currentDocumentId 
}) => {
  const [groups, setGroups] = useState<Group[]>([]);
  const [showNewGroupForm, setShowNewGroupForm] = useState(false);
  const [newGroupName, setNewGroupName] = useState('');
  const [showNewFileForm, setShowNewFileForm] = useState<string | null>(null);
  const [newFileName, setNewFileName] = useState('');
  const [collapsedGroups, setCollapsedGroups] = useState<Set<string>>(new Set());
  const [deleteConfirm, setDeleteConfirm] = useState<{
    type: 'group' | 'file';
    groupId: string;
    fileId?: string;
    name: string;
    isOpen: boolean;
  } | null>(null);

  useEffect(() => {
    // 从后端加载组和文件
    window.electronAPI.getGroups().then((data: any) => {
      setGroups(data.groups || []);
    });
  }, []);

  const handleCreateGroup = async () => {
    if (!newGroupName.trim()) return;
    try {
      const newGroup = await window.electronAPI.createGroup(newGroupName);
      setGroups([...groups, newGroup]);
      setNewGroupName('');
      setShowNewGroupForm(false);
    } catch (error) {
      console.error('Failed to create group:', error);
    }
  };

  const handleDeleteGroup = async (groupId: string) => {
    const group = groups.find(g => g.id === groupId);
    if (!group) return;

    setDeleteConfirm({
      type: 'group',
      groupId,
      name: group.name,
      isOpen: true
    });
  };

  const handleCreateFile = async (groupId: string) => {
    if (!newFileName.trim()) return;
    try {
      const newFile = await window.electronAPI.createFile(groupId, newFileName);
      setGroups(groups.map(group => {
        if (group.id === groupId) {
          return {
            ...group,
            files: [...group.files, newFile]
          };
        }
        return group;
      }));
      setNewFileName('');
      setShowNewFileForm(null);
      // 创建后自动选择新文件
      onDocumentSelect(groupId, newFile.id);
    } catch (error) {
      console.error('Failed to create file:', error);
    }
  };

  const handleDeleteFile = async (groupId: string, fileId: string) => {
    const group = groups.find(g => g.id === groupId);
    const file = group?.files.find(f => f.id === fileId);
    if (!group || !file) return;

    setDeleteConfirm({
      type: 'file',
      groupId,
      fileId,
      name: file.name,
      isOpen: true
    });
  };

  const confirmDelete = async () => {
    if (!deleteConfirm) return;

    try {
      if (deleteConfirm.type === 'group') {
        await window.electronAPI.deleteGroup(deleteConfirm.groupId);
        setGroups(groups.filter(g => g.id !== deleteConfirm.groupId));
      } else {
        if (!deleteConfirm.fileId) return;
        await window.electronAPI.deleteFile(deleteConfirm.groupId, deleteConfirm.fileId);
        setGroups(groups.map(group => {
          if (group.id === deleteConfirm.groupId) {
            return {
              ...group,
              files: group.files.filter(f => f.id !== deleteConfirm.fileId)
            };
          }
          return group;
        }));
      }
    } catch (err) {
      console.error(`Failed to delete ${deleteConfirm.type}:`, err);
    }

    setDeleteConfirm(null);
  };

  const toggleGroup = (groupId: string) => {
    setCollapsedGroups(prev => {
      const newSet = new Set(prev);
      if (newSet.has(groupId)) {
        newSet.delete(groupId);
      } else {
        newSet.add(groupId);
      }
      return newSet;
    });
  };

  return (
    <div className={`sidebar-container ${!isOpen ? 'collapsed' : ''}`}>
      <div className={`sidebar ${!isOpen ? 'collapsed' : ''}`}>
        <button className="add-group-button" onClick={() => setShowNewGroupForm(true)}>
          <FontAwesomeIcon icon={faFolderPlus} /> New Group
        </button>

        {showNewGroupForm && (
          <div className="new-item-form">
            <input
              type="text"
              value={newGroupName}
              onChange={(e) => setNewGroupName(e.target.value)}
              placeholder="Group name"
              onKeyPress={(e) => e.key === 'Enter' && handleCreateGroup()}
            />
            <button onClick={handleCreateGroup}>Create</button>
          </div>
        )}

        {groups.map(group => (
          <div key={group.id} className="group">
            <div className="group-header" onClick={() => toggleGroup(group.id)}>
              <span className="group-name">
                <FontAwesomeIcon 
                  icon={collapsedGroups.has(group.id) ? faChevronRight : faChevronDown} 
                  className="group-toggle-icon"
                />
                <FontAwesomeIcon icon={faFolder} /> {group.name}
              </span>
              <div className="group-actions">
                <button 
                  className="action-button" 
                  onClick={(e) => {
                    e.stopPropagation();
                    setShowNewFileForm(group.id);
                  }}
                >
                  <FontAwesomeIcon icon={faPlus} />
                </button>
                <button 
                  className="action-button" 
                  onClick={(e) => {
                    e.stopPropagation();
                    handleDeleteGroup(group.id);
                  }}
                >
                  <FontAwesomeIcon icon={faTrash} />
                </button>
              </div>
            </div>

            {showNewFileForm === group.id && (
              <div className="new-item-form">
                <input
                  type="text"
                  value={newFileName}
                  onChange={(e) => setNewFileName(e.target.value)}
                  placeholder="File name"
                  onKeyPress={(e) => e.key === 'Enter' && handleCreateFile(group.id)}
                />
                <button onClick={() => handleCreateFile(group.id)}>Create</button>
              </div>
            )}

            <div className={`group-files ${collapsedGroups.has(group.id) ? 'collapsed' : ''}`}>
              {group.files.map(file => (
                <div 
                  key={file.id} 
                  className={`file-item ${currentDocumentId === file.id ? 'active' : ''}`}
                  onClick={() => onDocumentSelect(group.id, file.id)}
                >
                  <span className="file-name">
                    <FontAwesomeIcon icon={faFile} /> {file.name}
                  </span>
                  <div className="file-actions">
                    <button
                      className="action-button"
                      onClick={(e) => {
                        e.stopPropagation();
                        handleDeleteFile(group.id, file.id);
                      }}
                    >
                      <FontAwesomeIcon icon={faTrash} />
                    </button>
                  </div>
                </div>
              ))}
            </div>
          </div>
        ))}
      </div>
      <button className="toggle-button" onClick={onToggle}>
        <FontAwesomeIcon icon={isOpen ? faChevronLeft : faChevronRight} />
      </button>

      <ConfirmDialog
        isOpen={!!deleteConfirm}
        title={`Delete ${deleteConfirm?.type === 'group' ? 'Group' : 'File'}`}
        message={`Are you sure you want to delete ${deleteConfirm?.type} "${deleteConfirm?.name}"? This action cannot be undone.`}
        onConfirm={confirmDelete}
        onCancel={() => setDeleteConfirm(null)}
      />
    </div>
  );
};

export default Sidebar;
